"""
Provides towncrier version information.
"""

# This file is auto-generated! Do not edit!
# Use `python -m incremental.update towncrier` to change this file.

from incremental import Version

__version__ = Version("towncrier", 21, 9, 0)
__all__ = ["__version__"]
